/*
Filename : OLED_LIB.C
author   : Fayaz Hassan
Date     : 16-02-2018
MCU      : AVR
Display  : OLED (I2C)
*/

// ============================================================================

//#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#define OLED_D	0b00001000
#define OLED_C	0b00010000

#define OLED_A		0x78	

#include "FONT_BASIC_16x8.H"

// ----------------------------------------------------------------------------

#define OLED_CH()(PORTB |= OLED_C )
#define OLED_CL()(PORTB &= (~OLED_C) )

#define OLED_DH()(PORTB |= OLED_D )
#define OLED_DL()(PORTB &= (~OLED_D) )

//==============================================================

const unsigned char  OLED_SC [] PROGMEM = 
{
0xAE, 0x20, 0b00, 0xB0, 0xC8, 0x00, 0x10, 0x40, 0x81, 0x3F, 0xA1, 0xA6, 0xA8,0x3F,
0xA4, 0xD3, 0x00, 0xD5, 0xF0, 0xD9, 0x22,0xDA, 0x12, 0xDB, 0x20, 0x8D, 0x14, 0xAF
};

//==============================================================

void OLED_STT(void)
{
	OLED_DH();
	OLED_CH();
	
	OLED_DL();
	OLED_CL();
}

//==============================================================

void OLED_STP(void)
{
	OLED_CL();
	OLED_DL();

	OLED_CH();
	OLED_DL();
}

//==============================================================

void OLED_BT(uint8_t byte)
{
	uint8_t b=byte;;
	for (uint8_t i = 0; i < 8; i++)
	{

		if ( b & 0b10000000 )
			{OLED_DH();}
		else
			{OLED_DL();}
		
		b = (b << 1);
		OLED_CH();
		OLED_CL();
	}
	OLED_DH();
	OLED_CH();
	OLED_CL();
}

//==============================================================

void OLED_CM(uint8_t cm)
{
	OLED_STT();
	OLED_BT(OLED_A);  
	OLED_BT(0x00);	// command
	OLED_BT(cm);
	OLED_STP();
}

//==============================================================

void OLED_CM2(uint8_t cm1, uint8_t cm2)
{
	OLED_STT();
	OLED_BT(OLED_A);  
	OLED_BT(0x00);	// command
	OLED_BT(cm1);
	OLED_BT(cm2);
	OLED_STP();
}

//==============================================================

void OLED_SD(uint8_t byt)
{
	OLED_STT();
	OLED_BT(OLED_A);
	OLED_BT(0x40);    //data
	OLED_BT(byt);
	OLED_STP();
}

//==============================================================

void OLED_init(void)
{
	DDRB |= (OLED_D | OLED_C);	
	
	for (uint8_t i = 0; i < sizeof (OLED_SC); i++) {
		OLED_CM(pgm_read_byte(&OLED_SC[i]));
	}
}

//==============================================================

void OLED_XY ( uint8_t x, uint8_t y )
{
	OLED_CM(0xb0+y);
	OLED_CM2(0x10 | ((x&0xf0)>>4),x & 0x0f);
}

//==============================================================

void OLED_FIL(uint8_t x) 
{

	OLED_CM(0xb0);
	OLED_CM2(0x10,0x01);
	OLED_STT();
	OLED_BT(OLED_A);
	OLED_BT(0x40);
	for ( uint8_t r=0; r<8; r++ )
	{
		for (uint8_t c = 0; c < 128; c++)
			OLED_BT(x);
	}
	OLED_STP();
}

//===========================================================

void OLED_SHST ( char *str   , uint8_t x, uint8_t y)
{
	for ( uint8_t n=0; n<2; n++ ) // to select row
	{
		OLED_XY ( x, n+y );
		for ( uint8_t i=0; str[i]!=0; i++ ) // to select char of str
		{
			uint8_t chr = str[i];
			uint16_t ch = (chr-' ')*8*2;
			OLED_STT();
			OLED_BT(OLED_A);
			OLED_BT(0x40);	//write data
			for ( uint8_t c=0; c<8; c++ )
			{
				OLED_BT(pgm_read_byte(&FONT_BASIC_16_8[ch+(n*8)+c]));
				if ( (str[i]=='.') && c>3) // check for dot
					break;
			}
			if ( chr=='M' || chr=='W' )
			{
				if (n==0)
					OLED_BT (0x02);
				else
					OLED_BT (0x20);
			}
			for ( uint8_t c=0; c<2; c++ ) // space between chars
			{
				OLED_BT(0);
			}

			OLED_STP();
		}
	}
}

//==============================================================

#include "OLED_NUMS_32x24.H"
void OLED_show_str_32x24 ( char *str , uint8_t slen, uint8_t x, uint8_t y)
{
	for ( uint8_t n=0; n<4; n++ ) // to select row
	{
		OLED_XY ( x, n+y );
		for ( uint8_t i=0; str[i]!=0 && i<slen; i++ ) // to select char of str
		{
			uint16_t ch = (str[i]-'.')*24*4;
			OLED_STT();
			OLED_BT(OLED_A);
			OLED_BT(0x40);	//write data
			for ( uint8_t c=0; c<24; c++ )
			{
				OLED_BT(pgm_read_byte(&OLED_FONT_32_24[ch+(n*24)+c]));
				if ( ch==0 && c>10 )
					break;
			}
			for ( uint8_t c=0; c<3; c++ ) // space between chars
			{
				OLED_BT(0);
			}

			OLED_STP();
		}
	}
}

//==============================================================

