/*
Filename : ZenerMeter.C
author   : Fayaz Hassan
Date     : 12-04-2018
MCU      : ATtiny85
Display  : OLED (I2C)

*/

#define F_CPU 1000000UL

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "OLED_LIB.C"

#define BZR 0b00000001 
int MXRG = 400;      

//========================================================================================
void ADC_init ( )
{
    ADCSRA |= ( (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) | (1 << ADEN) ); 
}

//========================================================================================

int RDADC ( int channelno )
{
	ADMUX = (channelno%4);
    ADCSRA |= (1 << ADSC);
    while (ADCSRA & (1 << ADSC));
	return ADC;
}

//========================================================================================

int main ( )
{
	int V=0;
	int A=0;
	char dsps[6]="00.0/";
	uint8_t nlp = 100;

	_delay_ms ( 50 );
	DDRB = BZR;
	
	PORTB |= BZR;
	_delay_ms ( 500 );
	PORTB &= (~BZR);

	OLED_init();
	_delay_ms ( 10 );
	
	OLED_FIL(0xFF);
	_delay_ms ( 500 );
	ADC_init ( );

	OLED_FIL(0x00);
	_delay_ms ( 500 );

	OLED_SHST ( "ZENER METER",11,3); 
	_delay_ms ( 1000 );

	OLED_FIL(0x00);
	_delay_ms ( 300 );
	RDADC ( 1 );
	RDADC ( 1 );
	
	while ( 1 )  
	{
		nlp++;
		
		A = RDADC ( 1 );
		_delay_ms ( 5 );
		A = RDADC ( 1 );
		
		V = A/2;
	
		if ( ( PINB & 0x02 ) == 0 )
			MXRG = 300;
		else
			MXRG = 400;

		OLED_SHST ( " ZENER METER" ,0, 0);
		if ( MXRG == 400 )
			OLED_SHST ( "0-40 VOLTS DC" ,0 ,6);
		else
			OLED_SHST ( "0-30 VOLTS DC" ,0 ,6);
		
		if ( V>MXRG )
		{
			OLED_SHST ( "ZENER ABSENT" , 0 ,2);
			if ( MXRG == 400 )
				OLED_SHST ( "OR ABOVE 40V" , 0 ,4);
			else
				OLED_SHST ( "OR ABOVE 30V" , 0 ,4);

			nlp=100;
			_delay_ms ( 200 ); //extra delay
		}
		else
		{
			if ( V < 10 && nlp>30 )
				nlp = 0;

			dsps[0]=V/100+'0';
			V=V%100;
			dsps[1]=V/10+'0';
			dsps[2]='.';
			dsps[3]=V%10+'0';
			dsps[4]='/'; //=V
			dsps[5]=0;

			if ( dsps[0]=='0' )
				dsps[0] = ':';
			OLED_show_str_32x24 ( dsps , 5, 0,2);
		}

		if (nlp<5)
			PORTB |= BZR;
		else
			PORTB &= (~BZR);
			
		if (nlp>100)
			nlp=100;


		_delay_ms ( 200 );


	} // end of while(1)

	return 0;
}


